var EntityHandler = {
    reflectEntity: function (runtime, entity) {
        var components = runtime.bridge.GetComponents(entity);

        components.forEach(function (typeId) {
            var typeName = runtime.bridge.getTypeName(typeId);
            var memberName = lowercaseFirstLetter(typeName);

            var component = ComponentHandler.wrapComponent(runtime, entity, typeName);

            entity[memberName] = component;
        });
    },
    createEntityFactory: function (runtime, type) {
        return function (name) {
            var self = Entity.create(runtime, type, name);

            EntityHandler.reflectEntity(runtime, self);

            return self;
        };
    },
    getEntityFactory: function (runtime, type) {
        return function (name) {
            var self = Entity.get(runtime, name, type);

            EntityHandler.reflectEntity(runtime, self);

            return self;
        };
    },
    initializeEntities: function (runtime) {
        var entityList = [
            "Camera",
            "Light",
            "Cube",
            "Sphere",
            "Cylinder",
            "Plane",
            "MeshPart",
            "Group",
            "Lod",
            "MeshPartWithMaterial",
            "ModelEntity",
            "SkyDome",
            "HeightMap",
            "GenericShape",
            "Trajectory",
            "DepthAxis",
            "AxisCube",
            "CubeMarker",
            "DataSet",
            "Extrusion",
            "Wellbore",
            "WellLog",
            "LookupColorMap",
            "Casing",
            "DataSet2D",
            "OceanRectangle",
            "Environment",
            "Billboard",
            "Overlay",
            "Text",
            "SimSalma_Simulator",
            "VolHorde",
            "VolDensityField"
        ];

        entityList.forEach(function (name) {
            runtime[name] = { create: EntityHandler.createEntityFactory(runtime, name) };
            runtime[name].get = EntityHandler.getEntityFactory(runtime, name);
        });

        runtime.Entity = {
            wrap: function (id) {
                var entity = Entity.wrap(runtime, id);

                EntityHandler.reflectEntity(runtime, entity);

                return entity;
            }
        }

        // Setup simple entity template Aliases
        runtime.Shape = runtime.GenericShape;
        runtime.Log = runtime.WellLog;
        runtime.ColorMap = runtime.LookupColorMap;
        runtime.BasicOcean = runtime.OceanRectangle;
    }
};
