#include "AdaptivePlanarGrid.hlsl"

#if defined(COGS_BASIC_TERRAIN_ELEVATION_IN_RED)
float elevation(float2 texpos)
{
  return Elevation.SampleLevel(ElevationSampler, texpos, 0.0).r;
}
#elif defined(COGS_BASIC_TERRAIN_ELEVATION_IN_ALPHA)
float elevation(float2 texpos)
{
  return Elevation.SampleLevel(ElevationSampler, texpos, 0.0).a;
}
#elif defined(COGS_BASIC_TERRAIN_ELEVATION_IN_RED_GREEN_BLUE)
float elevation(float2 texpos)
{
  float3 value = Elevation.SampleLevel(ElevationSampler, texpos, 0.0).rgb;
  return 256.0 * 256.0 * 256.0 * value.r + 256.0 * 256.0 * value.g + 256.0 * value.b;
}
#endif

VertexOut vertexFunction(VertexIn In)
{
  // Snap edge vertices to neighbouring grids.
  float2 pos = snapAdaptivePlanarGridEdges(In.Position.xyz).xy;

  // Get elevation.
  float2 texCoord = getTexCoordUnit(pos);
  float e_raw = elevation(texCoord);

  float e = elevationTransform.x * e_raw + elevationTransform.y;

  float4 surfacePositionWorld = mul(float4(pos, e, 1.f), worldMatrix);

  // Output results
  VertexOut vertex = (VertexOut)0;
  vertex.WorldPosition = surfacePositionWorld;
  vertex.Position = mul(surfacePositionWorld, worldToClipMatrix);
  vertex.texCoord = float3(texCoord, e_raw);

  return vertex;
}
