#include "Engine/Common.hlsl"

uint w;
uint h;


struct VertexIn
{
  float4 Position : SV_Position;
  float4 ScreenPosition : TEXCOORD0;
};

Texture2D colorTexture;
Texture2D depthTexture;

RWStructuredBuffer<float> packBuffer;

float4 main(VertexIn In, float4 ScreenPos : SV_Position) : SV_Target
{
  const float4 color = colorTexture.Load(int3(ScreenPos.xy, 0));
  const float depth = depthTexture.Load(int3(ScreenPos.xy, 0)).r;

  float4 viewSpacePosition = mul(float4(In.ScreenPosition.xy, depth, 1), inverseProjectionMatrix);
  viewSpacePosition /= viewSpacePosition.w;

  uint2 p = uint2(ScreenPos.xy);
  if (p.x < w && p.y < h) {
    packBuffer[p.y*w + p.x] = -viewSpacePosition.z;
  }

  return float4(color.rgb, 1);
}
