#ifndef COGS_COMMON_HLSL
#define COGS_COMMON_HLSL

cbuffer SceneBuffer
{
  float4x4 projectionMatrix;
  float4x4 viewMatrix;
  float4x4 inverseViewMatrix;
  float4x4 inverseProjectionMatrix;
  float4x4 worldToClipMatrix;
  float4 projectionParameters;
  float2 viewportOrigin;
  float2 viewportSize;
  float4 clippingPlanes[8];
  uint4 blueNoiseOffset;
  float shadowDepthClamp;
  float animationTime;
  float exposure;
  int submerged;
};

cbuffer ObjectBuffer
{
  float4x4 worldMatrix;
  uint objectId;
};

cbuffer AnimationBuffer
{
  float4x4 boneTransforms[256];
};

struct SurfaceOut
{
  float4 Color;
  float3 Specular;
  float SpecularPower;
  float3 Emissive;
  float3 Normal;

#ifdef COGS_CUSTOM_DEPTH_WRITE
  float Depth;
#endif
};

#ifndef VERTEX_FUNCTION
#define VERTEX_FUNCTION vertexFunction
#endif

#ifndef GEOMETRY_FUNCTION
#define GEOMETRY_FUNCTION geometryFunction
#endif

#ifndef SURFACE_FUNCTION
#define SURFACE_FUNCTION surfaceFunction
#endif

#define COGS_LIGHT_MASK_PHONG 1
#define COGS_LIGHT_MASK_INDIRECT 64
#define COGS_LIGHT_MASK_SHADOW_RECEIVER 128

int GetMaterialId()
{
  int materialId = 0;

#if defined(COGS_LIGHT_MODEL_PHONG)
  materialId |= COGS_LIGHT_MASK_PHONG;
#elif defined(COGS_LIGHT_MODEL_STANDARD)
  materialId |= 4;
#endif

#ifdef COGS_SURFACE_CLOSED
  materialId |= 16;
#endif

#ifdef COGS_INDIRECT_LIGHTING_FULL
  materialId |= COGS_LIGHT_MASK_INDIRECT;
#endif

#ifdef COGS_SHADOW_RECEIVER 
  materialId |= COGS_LIGHT_MASK_SHADOW_RECEIVER;
#endif

  return materialId;
}

#endif
