#include "scanDefines.hlsl"

cbuffer ReduceParameters
{
  uint baseLevel;
  uint inOffset;
  uint outOffset;
  uint gridWidth;
  uint N;
  uint B;
  uint inputWidth;
};

Texture2D<float> inputTexture;
RWStructuredBuffer<uint> scratch;

groupshared uint shmem[3 * SCAN_WARPSIZE];

[numthreads(SCAN_BLOCKSIZE, 1, 1)]
void main(uint3 threadIdx : SV_GroupThreadID,
          uint3 blockIdx : SV_GroupID)
{
  const uint t = threadIdx.x;
  const uint b = blockIdx.y*gridWidth + blockIdx.x;
  if (B <= b) {
    return;
  }
  const uint ix4 = SCAN_BLOCKSIZE*b + t;
  uint4 ix = ((uint4)(4 * ix4) + uint4(0, 1, 2, 3));
  
  uint sum;
  if (baseLevel) {
    if (ix.x < N) {
      uint4 i = ix % (uint4)inputWidth;
      uint4 j = ix / (uint4)inputWidth;
      if (ix.w < N) { // fast path, all in
        sum = inputTexture.Load(int3(i.x, j.x, 0))
            + inputTexture.Load(int3(i.y, j.y, 0))
            + inputTexture.Load(int3(i.z, j.z, 0))
            + inputTexture.Load(int3(i.w, j.w, 0));
      }
      else {  // Once per grid, some in
        sum = inputTexture.Load(int3(i.x, j.x, 0));
        if (ix.y < N) sum += inputTexture.Load(int3(i.y, j.y, 0));
        if (ix.z < N) sum += inputTexture.Load(int3(i.z, j.z, 0));
      }
    }
    else {
      sum = 0;
    }
  }
  else {
    if (ix.x < N) {
      const uint4 jx = (uint4)inOffset + ix;
      if (ix.w < N) {
        sum = scratch[jx.x] + scratch[jx.y] + scratch[jx.z] + scratch[jx.w];
      }
      else {
        sum = scratch[jx.x];
        if (ix.y < N) sum += scratch[jx.y];
        if (ix.z < N) sum += scratch[jx.z];
      }
    }
    else {
      sum = 0;
    }
  }

  const bool firstWarp = t < SCAN_WARPSIZE;

  if (!firstWarp) {
    shmem[t - SCAN_WARPSIZE] = sum;
  }
  GroupMemoryBarrierWithGroupSync();


#ifdef SCAN_WARP_LOCKSTEP
  if (firstWarp) {
    shmem[t] += sum + shmem[t + SCAN_WARPSIZE] + shmem[t + 2 * SCAN_WARPSIZE];;
    GroupMemoryBarrier();

    [unroll]
    for (uint r = SCAN_WARPSIZE_L2 - 1; r > 0; r--) {
      const uint w = (1 << r);
      if (t < w) {
        uint a = shmem[t] + shmem[t + w];
        GroupMemoryBarrier();
        shmem[t] = a;
        GroupMemoryBarrier();
      }
    }

    if (t == 0) {
      scratch[outOffset + b] = shmem[0] + shmem[1];
    }
  }
#else
  if (firstWarp) {
    sum += shmem[t] + shmem[t + SCAN_WARPSIZE] + shmem[t + 2 * SCAN_WARPSIZE];
    shmem[t] = sum;
  }
  GroupMemoryBarrierWithGroupSync();

  [unroll]
  for (uint r = SCAN_WARPSIZE_L2 - 1; r > 0; r--) {
    const uint w = (1 << r);
    uint a = 0;
    if (t < w) {
      a = shmem[t] + shmem[t + w];
    }
    GroupMemoryBarrierWithGroupSync();  // necessary?
    if (t < w) {
      shmem[t] = a;
    }
    GroupMemoryBarrierWithGroupSync();
  }

  if (t == 0) {
    scratch[outOffset + b] = shmem[0] + shmem[1];
  }
#endif
}
