#ifdef COGS_DITHER_DEFERRED
#include "BlueNoise.hlsl"
#endif

struct PixelOut
{
  float4 A : SV_Target0;  // 4 x uchar
  float4 B : SV_Target1;  // 4 x uchar
  float4 C : SV_Target2;  // 4 x uchar
  float4 D : SV_Target3;  // 4 x half
};

PixelOut main(VertexIn In, float4 ScreenPos : SV_Position)
{
#ifdef SURFACE_FUNCTION
  SurfaceOut surface = SURFACE_FUNCTION(In);
#else
  SurfaceOut surface = surfaceFunction(In);
#endif

#ifdef COGS_ALPHA_TEST_DISCARD
  if (surface.Color.a < alphaThreshold) discard;
#endif

  PixelOut Out = (PixelOut)0;
  Out.A = float4(surface.Color);
  Out.B = float4(surface.Emissive, 1);
  
  Out.C = float4(surface.Specular, (1.0 / 255.0) * GetMaterialId());
  Out.D = float4(surface.Normal, surface.SpecularPower);

#ifdef COGS_DITHER_DEFERRED
  float3 dither = (blueNoise3(int2(ScreenPos.xy))*2.0-1.0)*(2.0/255);
  Out.A.rgb = saturate(Out.A.rgb + dither);
  Out.B.rgb = saturate(Out.B.rgb + dither);
  if(max(max(Out.C.r, Out.C.g), Out.C.b) > 0.0)
    Out.C.rgb = saturate(Out.C.rgb + dither);
#endif

  return Out;
}
