uniform samplerCube cubeShadowMap;

float vectorToDepth(in vec3 v, in float n, in float f)
{
  vec3 absVector = abs(v);
  float localViewZ = max(absVector.x, max(absVector.y, absVector.z));

  float normalizedZ = (f + n) / (f - n) + ((-2.0 * f * n) / (f - n)) * (1.0 / localViewZ);
#ifdef COGS_REVERSE_DEPTH
  normalizedZ = -normalizedZ;
#endif

  return normalizedZ*0.5 + 0.5;
}

float calculatePointLightShadow(in vec4 worldLightPos, in vec3 worldPosition, in vec3 normal)
{
  vec3 toLight = worldLightPos.xyz - worldPosition;
  vec3 lightDir = normalize(toLight);
   
  vec3 shadowCoords = -lightDir;// +samplingDisk[j] * texelSize

  float depth = vectorToDepth(toLight,
                              shadowData[POINT_SHADOW_NEAR_FAR].x,
                              shadowData[POINT_SHADOW_NEAR_FAR].y);

  float shadowFactor = textureCube(cubeShadowMap, shadowCoords.xzy).r < depth  ? 0.0 : 1.0;

  return shadowFactor;
}
