#define COGS_COMBINE(a, b) a##b

vec3 GetNormal(in VertexIn In)
{
#ifdef COGS_STANDARD_MATERIAL_NORMAL_MAP
  vec3 normal = normalize(In.Normal);
  vec3 normalColor = 2.0 * texture2D(normalMap, COGS_COMBINE(In.TexCoord, COGS_STANDARD_MATERIAL_NORMAL_UV)).rgb - vec3(1);

  vec3 tangent = normalize(In.Tangent);

#ifdef COGS_STANDARD_MATERIAL_HAS_BITANGENT
  vec3 bitangent = normalize(In.Bitangent);
#else
  vec3 bitangent = normalize(cross(normal, tangent));
#endif

  vec3 bumpNormal = normalize(normalColor.z * normal + normalColor.x * tangent + normalColor.y * bitangent);
  normal = normalize(mix(normal, bumpNormal, normalMapFactor));
  
  return In.IsFrontFace ? normal : -normal;
#else
  return normalize(In.IsFrontFace ? In.Normal : -In.Normal);
#endif
}

vec4 GetAlbedo(in VertexIn In)
{
#ifdef COGS_STANDARD_MATERIAL_ALBEDO_MAP
  return albedo() * texture2D(albedoMap, COGS_COMBINE(In.TexCoord, COGS_STANDARD_MATERIAL_ALBEDO_UV).xy);
#else
  return albedo();
#endif
}

float GetRoughness(in VertexIn In)
{
#ifdef COGS_STANDARD_MATERIAL_ROUGHNESS_MAP
  return roughness() * texture2D(roughnessMap, COGS_COMBINE(In.TexCoord, COGS_STANDARD_MATERIAL_ROUGHNESS_UV)).r;
#elif defined(COGS_STANDARD_MATERIAL_METALLIC_ROUGHNESS_MAP)
  return roughness() * texture2D(metallicRoughnessMap, COGS_COMBINE(In.TexCoord, COGS_STANDARD_MATERIAL_METALLIC_ROUGHNESS_UV)).g;
#else
  return roughness();
#endif
}

float GetMetallic(in VertexIn In)
{
#ifdef COGS_STANDARD_MATERIAL_METALLIC_MAP
  return metallic() * texture2D(metallicMap, COGS_COMBINE(In.TexCoord, COGS_STANDARD_MATERIAL_METALLIC_UV)).r;
#elif defined(COGS_STANDARD_MATERIAL_METALLIC_ROUGHNESS_MAP)
  return metallic() * texture2D(metallicRoughnessMap, COGS_COMBINE(In.TexCoord, COGS_STANDARD_MATERIAL_METALLIC_ROUGHNESS_UV)).b;
#else
  return metallic();
#endif
}

float GetGloss(in VertexIn In)
{
#ifdef COGS_STANDARD_MATERIAL_GLOSS_MAP
  return gloss() * texture2D(glossMap, COGS_COMBINE(In.TexCoord, COGS_STANDARD_MATERIAL_GLOSS_UV)).r;
#elif defined(COGS_STANDARD_MATERIAL_SPECULAR_GLOSS_MAP)
  return gloss() * texture2D(specularGlossMap, COGS_COMBINE(In.TexCoord, COGS_STANDARD_MATERIAL_SPECULAR_GLOSS_UV)).a;
#else
  return gloss();
#endif
}

vec3 GetSpecular(in VertexIn In)
{
#ifdef COGS_STANDARD_MATERIAL_SPECULAR_MAP
  return specular() * texture2D(specularMap, COGS_COMBINE(In.TexCoord, COGS_STANDARD_MATERIAL_SPECULAR_UV)).rgb;
#elif defined(COGS_STANDARD_MATERIAL_SPECULAR_GLOSS_MAP)
  return specular() * texture2D(specularGlossMap, COGS_COMBINE(In.TexCoord, COGS_STANDARD_MATERIAL_SPECULAR_GLOSS_UV)).rgb;
#else
  return specular();
#endif
}

vec3 GetEmissive(in VertexIn In)
{
#ifdef COGS_STANDARD_MATERIAL_EMISSIVE_MAP
  return emissive() * texture2D(emissiveMap, COGS_COMBINE(In.TexCoord, COGS_STANDARD_MATERIAL_EMISSIVE_UV).xy).rgb;
#else
  return emissive();
#endif
}

float GetOcclusion(in VertexIn In)
{
#ifdef COGS_STANDARD_MATERIAL_OCCLUSION_MAP
  return occlusion() * texture2D(occlusionMap, COGS_COMBINE(In.TexCoord, COGS_STANDARD_MATERIAL_OCCLUSION_UV)).r;
#else
  return occlusion();
#endif
}
